/*****************************************************************************
   LICENSE
   Copyright (C) 2018 openQCM
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see http://www.gnu.org/licenses/gpl-3.0.txt
  --------------------------------------------------------------------------------
   
   OPENQCM Q-1 - Quartz Crystal Microbalance with dissipation monitoring 
   openQCM is the unique opensource quartz crystal microbalance http://openqcm.com/
   ELECTRONICS
     - board and firmware designed for teensy 3.6 development board
     - DDS/DAC Synthesizer AD9851
     - phase comparator AD8302
     - I2C digital potentiometer AD5251+
     - MCP9808 temperature sensor

   FEATURES add:
   - get ID board by reading EEPROM

   DEVELOPMENT version beta 4.0:
   - read temperature sensor only at the end of the sweep / scan procedure 
   - replace standard i2c library with Teensy i2c custom library i2c_t3.h
    - replace the original Adafruit_MCP9808 library with the one I modify by myself to work with teensy (just replace Wire.h with i2c_t3.h)
   - change the number of average AVERAGE_COUNT = 100 (TO_TEST)

   version  beta-4.0
   date     2018 july
   author   marco mauro openQCM team
   
   credit:
   based on the work made by Brett Killion on Hackaday
   https://hackaday.io/project/10021-arduino-network-analyzer
   Teensy 3.6 is developed developed by Paul Stoffregen working at PJRC
   https://www.pjrc.com/store/teensy36.html
   Brian (nox771) New I2C library for Teensy3 
   https://github.com/nox771/i2c_t3
   
 *****************************************************************************/

/************************** LIBRARIES **************************/


#include "src/i2c_t3_TEENSY.h"
#include "src/Adafruit_MCP9808_TEENSY.h"
#include <EEPROM.h>

/*************************** DEFINE ***************************/
// potentiometer AD5252 I2C address is 0x2C(44)
#define ADDRESS 0x2C
// potentiometer AD5252 default value
#define  POT_VALUE 254
// Teensy 3.6 ADC bit resolution value
#define ADC_RESOLUTION  13

// TODO change the number of averaging cycle AVERAGE_COUNT = 100
// number of averaging cycles
#define AVERAGE_COUNT 100.0
// reference clock
#define REFCLK 125000000
// current address in EEPROM series
#define ADDRESS_SERIES 0
// current address in EEPROM first number
#define ADDRESS_NUMBERFIRST 1
// current address in EEPROM second number
#define ADDRESS_NUMBERSECOND 2

/*************************** VARIABLE DECLARATION ***************************/
// input character array
char incmd[10];
// index input character array
int cindex = 0;
// current input frequency
long freq = 0;
// input character
char ch;

// init previous frequency and current frequency variable
// previous frequency value
long freq_pre = 0;
// current frequency value 
long freq_now = 0;

// DDS Synthesizer AD9851 pin function
int WCLK = A8;
int DATA = A9;
int FQ_UD = A1;

// frequency tuning word
long FTW;
float temp_FTW; // temporary variable

// phase comparator AD8302 pinout
int AD8302_PHASE = 20;
int AD8302_MAG = 37;
//int AD83202_REF = 17;
int AD83202_REF = 34;


// phase AD8302 variable
int phase_t = 0;
// mean value
double phase_m;
// magnitude AD8302 variable
int mag_t = 0;
// mean value
double mag_m = 0;
// log amplitude AD8302 variable
int ampli_t = 0;
// mean value
double ampli;

double val = 0;

// Create the MCP9808 temperature sensor object
Adafruit_MCP9808 tempsensor = Adafruit_MCP9808();
// init temperature variable
float temperature = 0;
// reading temperature boolean flag
boolean readingTemperature = false;


// LED pin
int ledPin1 = 24;
int ledPin2 = 25;

// start boolean
boolean startFlag = false;
// averaging boolean
boolean AVERAGE = true;

/*************************** FUNCTION ***************************/
// AD9851 power off function
void powerOff()
{
  long pointer = 1;
  int pointer2 = 0b10000000;
  int lastByte = 0b10100000;

  digitalWrite(DATA, LOW);

  for (int i = 0; i < 32; i++)
  {
    digitalWrite(WCLK, HIGH);
    digitalWrite(WCLK, LOW);
  }
  for (int i = 0; i < 8; i++)
  {
    if ((lastByte & pointer2) > 0) digitalWrite(DATA, HIGH);
    else digitalWrite(DATA, LOW);
    digitalWrite(WCLK, HIGH);
    digitalWrite(WCLK, LOW);
    pointer2 = pointer2 >> 1;
  }
  digitalWrite(FQ_UD, HIGH);
  digitalWrite(FQ_UD, LOW);
}

/* AD9851 set frequency fucntion */
void SetFreq(long frequency)
{
  // FTW = (frequency * pow(2, 24)) / (179999563 / 256);
  // set to 125 MHz internal clock
  temp_FTW = (frequency * pow(2, 32)) / REFCLK;
  FTW = long (temp_FTW);

  long pointer = 1;
  int pointer2 = 0b10000000;
  int lastByte = 0b10000000;

  /* 32 bit dds tuning word frequency instructions */
  for (int i = 0; i < 32; i++)
  {
    if ((FTW & pointer) > 0) digitalWrite(DATA, HIGH);
    else digitalWrite(DATA, LOW);
    digitalWrite(WCLK, HIGH);
    digitalWrite(WCLK, LOW);
    pointer = pointer << 1;
  }

  /* 8 bit dds phase and x6 multiplier refclock*/
  for (int i = 0; i < 8; i++)
  {
    //if ((lastByte & pointer2) > 0) digitalWrite(DATA, HIGH);
    //else digitalWrite(DATA, LOW);
    digitalWrite(DATA, LOW);
    digitalWrite(WCLK, HIGH);
    digitalWrite(WCLK, LOW);
    pointer2 = pointer2 >> 1;
  }

  digitalWrite(FQ_UD, HIGH);
  digitalWrite(FQ_UD, LOW);

  //FTW = 0;
}

/* measure and print magnitude, phase and temperatuer */
void measurePrint(boolean isTemperatureRead) {
  
  // read AD8307 power detector via teesny ADC
  // reading ADC several times
  for (int i = 0; i < AVERAGE_COUNT; i++ ) {
    int measure_phase = analogRead(AD8302_PHASE);
    int measure_mag = analogRead(AD8302_MAG);
    // store the new ADC readings
    phase_t = phase_t + measure_phase;
    mag_t = mag_t + measure_mag;
  }

  // averaging ADC readings
  phase_m = phase_t / AVERAGE_COUNT;
  // clear variable
  phase_t = 0;
  mag_m = mag_t / AVERAGE_COUNT;
  // clear variable
  mag_t = 0;
  
  // print to serial using string format for ardulink listener
  if (isTemperatureRead == false){
    Serial.print("RAWMONITOR");
    Serial.print((int) (phase_m * 100) ) ;
    Serial.print("_");
    Serial.print((int) (mag_m * 100) );
    Serial.write(255);
  }
  else if (isTemperatureRead == true){
    Serial.print("RAWMONITOR");
    Serial.print((int) (phase_m * 100) ) ;
    Serial.print("_");
    Serial.print((int) (mag_m * 100) );
  }

}

/*************************** SETUP ***************************/
void setup()
{
  // Initialise I2C communication as Master
  Wire.begin();
  // Initialise serial communication, set baud rate = 115200
  Serial.begin(115200);

  // set potentiometer value
  // Start I2C transmission
  Wire.beginTransmission(ADDRESS);
  // Send instruction for POT channel-0
  Wire.write(0x01);
  // Input resistance value, 0x80(128)
  Wire.write(POT_VALUE);
  // Stop I2C transmission
  Wire.endTransmission();

  // AD9851 set pin mode
  pinMode(WCLK, OUTPUT);
  pinMode(DATA, OUTPUT);
  pinMode(FQ_UD, OUTPUT);
  // AD9851 enter serial mode
  digitalWrite(WCLK, HIGH);
  digitalWrite(WCLK, LOW);
  digitalWrite(FQ_UD, HIGH);
  digitalWrite(FQ_UD, LOW);

  // AD8302 set pin mode
  pinMode(AD8302_PHASE, INPUT);
  pinMode(AD8302_MAG, INPUT);
  pinMode(AD83202_REF, INPUT);

  // Teensy 3.6 set adc resolution
  analogReadResolution(ADC_RESOLUTION);

  // begin temperature sensor
  tempsensor.begin();
  
  // turn on the light
  pinMode(ledPin1, OUTPUT);
  pinMode(ledPin2, OUTPUT);
  digitalWrite(ledPin1, HIGH);
  digitalWrite(ledPin2, HIGH);
}

/*************************** LOOP ***************************/
void loop()
{
  // read message at serial port
  if (Serial.available()) {
    // cast the serial read value to integer
    freq = Serial.parseInt();
    // store the previous value of freq
    freq_pre = freq_now; 
    
    /* SWEEP SCAN COMMAND 
     * set the current frequency and instruct the DDS 
     * measure and print raw data 
     */  
       
     if ( ((freq > 10) && (freq <= 72000000)) && (startFlag) ) {
      
      // store the current value of frequency 
      freq_now = freq; 
      
      // check the pre and now freq 
      // scan / sweep procedure do not measure temperature
      if (freq_now >= freq_pre){
        // set temperature sensor flag
        readingTemperature = false;
        // set the current frequency
        SetFreq(freq);
        // measure phase gain and print
        measurePrint(readingTemperature);     
      }
      else if (freq_pre > freq_now){
        // measure temperarure 
        // set temperature sensor flag
        readingTemperature = true;
        
        // Don't remove this line! required before reading temp
        tempsensor.shutdown_wake(0);
        // measure temperature
        temperature = tempsensor.readTempC();
        
        // set the current frequency
        SetFreq(freq);
        // measure phase gain and temperature
        measurePrint(readingTemperature);     
        // print the data
        Serial.print("_");
        Serial.print((int)(temperature*100));
        // print end of message 
        Serial.write(255);
      }
      
    }

    /* STATE COMMAND 
     * 0 = IDLE state command, do nothing  
     * 1 = START measurement loop command
     * 2 = read ID in EEPROM command, return device ID 
     */
    
    // idle state command, do nothing
    else if (freq == 0) {
      Serial.print("IDLE");
      Serial.write(255);
      // probabably you should wait for a while
      delay(500);
      // set start flag to false
      startFlag = false;
    }

    // start measure command, start the sweep or scan procedure and measure
    else if (freq == 1) {
      Serial.print("START");
      Serial.write(255);
      // wait for a while
      delay(500);
      // set the start flag true 
      startFlag = true;
    }
    
    // read ID board in EEPROM and send ID to sw
    else if (freq == 2) {
      byte valueSeries = EEPROM.read(ADDRESS_SERIES);
      byte valueNumberFirst = EEPROM.read(ADDRESS_NUMBERFIRST);
      byte valueNumberSecond = EEPROM.read(ADDRESS_NUMBERSECOND);
      Serial.print("SERIALNUMBER");
      Serial.print(valueSeries, DEC);
      Serial.print(valueNumberFirst, DEC);
      Serial.print(valueNumberSecond, DEC);
      Serial.write(255);
    }
  }
}





